#include "compiler.h"
#include "bin2abc.h"

#define MAXLINE 79

int writebas(FILE * out, uint8_t * data, const bool * isset,
             int origin, int entrypoint, int firstline,
	     bool subroutine, const char *chain)
{
    int nextaddr = 0;
    int addr;
    char buf[MAXLINE+1];
    int buf_len = 0;

    if (have_relocs)
	apply_relocs(data, origin);

    for (addr = 0; addr < MAX_ADDR; addr++) {
        if (isset[addr]) {
            if (buf_len) {
                if (addr != nextaddr || buf_len > MAXLINE-5) {
		    buf[buf_len++] = '\n';
		    fwrite(buf, 1, buf_len, out);
		    buf_len = 0;
		}
	    }

	    if (!buf_len) {
		buf_len = snprintf(buf, sizeof buf, "%d POKE %d%%",
				   firstline++, addr);
	    }

	    buf_len += snprintf(buf + buf_len, sizeof buf - buf_len,
				",%d%%", data[addr]);
	    nextaddr = addr + 1;
	}
    }

    if (buf_len) {
	buf[buf_len++] = '\n';
	fwrite(buf, 1, buf_len, out);
    }

    if (subroutine) {
	fprintf(out, "%d RETURN\n", firstline++);
    } else {
	if (entrypoint >= 0)
	    fprintf(out, "%d Z%%=CALL(%d%%)\n", firstline++, entrypoint);

	if (chain[0] == ':') {
	    fprintf(out, "%d END\n", firstline++);
	} else if (!chain[0]) {
	    /* Make it work on both ABC80 and 800 */
	    fprintf(out, "%d ON ERROR GOTO %d\n", firstline, firstline+2);
	    fprintf(out, "%d CHAIN \"NUL:\"\n", firstline+1);
	    fprintf(out, "%d CHAIN \"\"\n", firstline+2);
	    firstline += 3;
	} else {
	    fprintf(out, "%d CHAIN \"%s\"\n", firstline++, chain);
	}
    }

    return 0;
}
